/* IluJava.h */
/* Chris Jacobi, April 23, 1997 9:28 am PDT */

/*
 * Copyright (c) 1996 Xerox Corporation.  All Rights Reserved.  
 * Unlimited use, reproduction, and distribution of this software is
 * permitted.  Any copy of this software must include both the above
 * copyright notice of Xerox Corporation and this paragraph.  Any
 * distribution of this software must comply with all applicable United
 * States export control laws.  This software is made available AS IS,
 * and XEROX CORPORATION DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
 * INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE, AND NOTWITHSTANDING ANY OTHER
 * PROVISION CONTAINED HEREIN, ANY LIABILITY FOR DAMAGES RESULTING FROM
 * THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 * CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 * XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
 
/* $Id: IluJava.h,v 1.19 1997/04/23 17:14:23 jacobi Exp $ */


#ifndef _ILUJAVA_EXPORTS_
#define _ILUJAVA_EXPORTS_

/*
 * Header file for ILU's Java implementatin.
 *
 * This header file is mostly there to support internal 
 * file to file communication of the java language run time
 * support.  Presence of an identifier does not make this
 * identifier conceptionally public.
 */


/* ================ Generic conventions ================ */


/* Other identifier (variable, field, procedures) conventions
 *
 * jj... thing is java object
 *     (2 j's because JNI types start with one j)
 *
 * c... thing is on c side 
 * secondary:	c... if meta object -> for ilu kernel's object system
 * EVENTUALLY SWITCH TOWARDS k for similarity with other languages
 *
 * y... thing is c pointer stored in a (long) java object 
 *
 * _ilujava_...	 regular prefix for anything which is not exported
 *        	 ouside the language specific runtime  (But it is  
 *        	 in the .h file for internal reasons)
 * _iluJava_...	 prefix for export to stubs level but not java or clients
 *        	 (Doesn't yet occur)
 *
 * xerox_ilu_... prefix for exported to Java (given by java stubber)
 * 
 * ilu_		 ilu kernel defined feature
 *
 * Identifiers which don't follow any conventon are either used very local,
 * or they might be defined by a Java system header file. 
 */
 
 
/* File name conventions
 * I have not yet thought about systems which require short file names
 *
 * ...java      	manually writen source file
 *   			(ILU stubs should not be part of this directory)
 * IluJava...c  	manually writen source file
 * IluJava...h  	manually writen source file
 * xerox_ilu_...c	generated by java native code stubber
 * xerox_ilu_...h	generated by java native code stubber
 */
 


/* ================ Imports etc ================ */


#include <StubPreamble.h>
#include <interpreter.h>
#include <javaString.h>
#include <iluxport.h>
#include <iluerror.h>
#include "IluJava_CJ.h"
#include "IluJava_JTypes.h"


#define EXPORTJAVA extern
/* Documents: following procedure implements a native java method */ 

#define INTERNALONLY static
/* Documents: following procedure is not used outside this file */

#define EXPORTLIMITED extern
/* Documents: following procedure is exported to other IluJava file 
 * (This is not applied religiously in .h files)
 */
 
#define EXPORTLIMITEDVAR 
/* Documents: following VARIABLE is implemented here but used also in 
 * other IluJava file 
 * (This is not used in .h files which must declare variable to be extern)
 */

#define IMPORTVAR extern
/* Documents: following VARIABLE is declared in other IluJava file 
 * (This is not used in .h files because all variable are extern anyway)
 */
 
#define OR ||
#define AND &&
 
/* ================ debugging ================ */


/* Available debugging flags (without any deep significance).
 *    0 means don't write to the log. 
 *    1 means log the interesting cases.
 *    Larger values of interest only while debugging a particular 
 *    feature (can cause excessive output).
 */
extern int _ilujava_iluGenericFlag; /* debugging mode */
extern int _ilujava_iluSpare1Flag;
extern int _ilujava_iluSpare2Flag;
extern int _ilujava_objectsFlag;
extern int _ilujava_pickleFlag;
extern int _ilujava_callsFlag;
extern int _ilujava_definitionsFlag;
extern int _ilujava_pctFlag;
extern int _ilujava_lockFlag;
extern int _ilujava_gcFlag;
extern int _ilujava_alarmFlag;


extern int _ilujava_getIntProp(char *, int);
/* Upcall to java to read an int property value as a means
 * to implement more debugging flags.
 * String is name of property (owned by caller). 
 * int is default value.
 */

#define BUGPRINT	_ilujava_BUG
/* Use this to print debugging output in an error situation. 
 * Also sets up a java exception.
 */

extern void _ilujava_BUG(char * f);
/* Report an error condition.
 *
 * Used for condition which are caused by bugs in the application or ilu  
 * for which further progress is however still possible.
 *
 * In production mode:
 * Prints error message and sets up a Java exception for raising on
 * return into Java.
 *
 * In debugging mode:
 * "stop" immediately to ease debugging.
 */
 

extern void _ilujava_PANIC(char * f);
/* 
 * Report error conditions for which ilu can not recover.
 */


extern char * _ilujava_captureJavaStack();
/* Debugging aid...
 * Caller takes ownership of returned string.
 * Rather expensive.
 */



/* ================ memory allocation ================ */


#include "IluJava_JMem.h"

extern char * _ilujava_notUsed1;
extern char * _ilujava_notUsed2;
/* Used to fool Java garbage collector into believing a pointer is
 * still in use:  Assign pointer to this to prevent C compiler from discarding 
 * pointer -> so Java garbage collector will find the reference on the 
 * stack...
 */


/* ================ garbage collection ================ */

/* Stuffing Java refs into the C heap is not extremely
 * portable. These macros or routines should make this
 * undertaking a little bit more portable.
 *
 * In Sun's JDK these are no-ops; nevertheless the 
 * REVEAL operation carries the risk of accessing
 * an object which has already been collected.
 *
 * With the Boehm collector, these will be real.
 * I do not yet understand the fine details of the
 * Microsoft Java implementation.
 * I don' know whether the Netscape implementation
 * is compatible with...
 */

#define DISGUISE_LSO(x) (x)
#define REVEAL_LSO(x) (x)
 
/* The LSR keeps the invariant that a LSO is not garbage collected
 * while the kernel hangs on to a KO-LSO pair.  This pair is
 * modified only with the server lock held.
 *
 * Note that the invariant is about garbage collection, not about
 * finalization.  All thinkable implementation with finalization
 * will provide the further invariant that an object is not garbage 
 * collected before it is finalized.
 */

/* 
================ exception handling ================ */

#define completedYes "^Y"
#define completedNo "^N"

extern void 
_ilujava_IluErrorToException(ilu_Error* errPtr, char * t);
/* Causes the java system exception which fits the ilu_Error
 * to be raised.  
 * This procedure returns! Exceptions are a Java only concept.  
 * Java system exceptions put special meaning to t if it
 * starts with the character "^".  
 * Exceptions subclass of IluSystemException.
 *
 * This always raises an exception:  Even ILU_SUCCESS_CASE !!
 * Explicitely also supports errPtr==0.  
 */

extern void 
_ilujava_IluErrorToCallException(ilu_Error* errPtr, char * t);
/* Like IluErrorToException except that there is no switch
 * on the exception but allways IluCallException is raised.
 */


extern void 
_ilujava_Report(ilu_Error* errPtr, char* t);
/* OPTIONALLY (according to command line options)
 * makes a call to ilu_DebugPrintf.
 * Does not raise any errors or exceptions.
 * Explicitely also supports errPtr==0.
 */

extern void
_ilujava_NullPointerException(char* t);
/* Causes a NullPointerException to be reported when returned into java.
 */
 
 
extern void
_ilujava_SignalInconsistency(char* t);
/* Causes a serious internal error to be reported when returned into java.
 */


/* ================ miscellanious ================ */


#define JAVALANGIDX _ilujava_LangIdx()
/* Access to ilu's per language index */

extern ilu_LanguageIndex _ilujava_LangIdx();
/* For implementation of JAVALANGIDX 
 * (Currently implemented in IluJava_IluDebug.c)
 */


extern void
_ilujava_registerJavaClass (
	ilu_Class cIluClass, 
	ilu_refany jClassRep   /* struct Hxerox_ilu_IluClassRep* */ 
	);
/* Register the mapping from ilu_Class to IluClassRep.
 * Implemented in IluJava_IluRT0.c
 * Used in IluJava_IluClassRep.c
 * Locked in IluJava_IluClassRep.c
 */


extern ilu_refany   /* struct Hxerox_ilu_IluClassRep* */
_ilujava_findJavaClass (
	ilu_Class cIluClass
	);
/* Find the IluClassRep for an ilu_Class.
 */


extern ilu_refany /* struct Hxerox_ilu_IluOInt* */
_ilujava_creatSurrOIFromRegistry (
	ilu_Object cIluObject, 
	ilu_refany jStaticClass   /* struct Hxerox_ilu_IluClassRep* */ 
	);
/* Register a surrogate class.
 * Implemented in IluJava_IluRT0.c
 * Used in IluJava_IluCall.c
 *
 * Sole reason for crummy refany is that we don't want this file to
 * depend on Hxerox_ilu_IluClassRep.h
 */


extern ilu_Passport 
_ilujava_getIluPassport(
	ilu_refany jIluPassport /* struct Hxerox_ilu_IluPassport* */ 
	);
/* Converts java passport to ilu passport.
 * Converts null IluPassport into default passport. 
 * Implemented in IluJava_IluPassport.c
 * Used in IluJava_Ilucall.c
 */

extern void
_ilujava_forkConnectionHandler(ilu_Connection newConnection);


/* ================ initialization ================ */

extern void _ilujava_IluRt0InitHashTable();
/* Initialization for IluJava_IluRT0 */

extern void _ilujava_AlarmTechInit();
/* Initialization for IluJava_AlarmTech */

extern void _ilujava_LockTechInit();
/* Initialization for IluJava_LockTech */

extern void _ilujava_IluOIntInit();
/* Initialization for IluJava_IluOInt */

extern void _ilujava_WaitTechInit();
/* Initialization for IluJava_pollwt.c */

extern void _ilujava_PassportInit();
/* Initialization for IluJava_IluPassport.c */

extern void _ilujava_IluServerInit();
/* Initialization for IluJava_IluServer.c */


/* ================ end ================ */

#endif /* _ILUJAVA_EXPORTS_ */


