/* IluJava_IluPort.c */ 
/* Chris Jacobi, April 10, 1997 9:30 am PDT */

/*
 * Copyright (c) 1996, 1997 Xerox Corporation.  All Rights Reserved.  
 * Unlimited use, reproduction, and distribution of this software is
 * permitted.  Any copy of this software must include both the above
 * copyright notice of Xerox Corporation and this paragraph.  Any
 * distribution of this software must comply with all applicable United
 * States export control laws.  This software is made available AS IS,
 * and XEROX CORPORATION DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
 * INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE, AND NOTWITHSTANDING ANY OTHER
 * PROVISION CONTAINED HEREIN, ANY LIABILITY FOR DAMAGES RESULTING FROM
 * THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 * CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 * XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
 
/* $Id: IluJava_IluPort.c,v 1.15 1997/04/14 22:41:08 jacobi Exp $ */
 
/*
 * C side of IluPort.java, IluPassport.java and IluTransportInfo.java
 */

#include <iluxport.h>
#include <iluerror.h>

#include "IluJava_JStubs.h"
#include "xerox_ilu_IluPassport.h"
#include "IluJava.h"


EXPORTJAVA void 
xerox_ilu_IluPort_nativeInitPort(
		struct Hxerox_ilu_IluPort * jjPort
		)
{
    struct Hxerox_ilu_IluServer * jjServer 
    	= unhand(jjPort)->jjServer;
    struct Hjava_lang_String * jjProtocolInfo 
    	= unhand(jjPort)->jjProtocolInfo;
    struct Hxerox_ilu_IluTransportInfo * jjTransportInfo 
    	= unhand(jjPort)->jjTransportInfo;
    struct Hxerox_ilu_IluPassport * jjPassport 
    	= unhand(jjPort)->jjPassport;
    ilu_string cProtInfo;
    ilu_Server cServer;
    ilu_TransportInfo cTransInfo = 0;
    ilu_Passport cPassport = 0; 
    ilu_Port cPort;
    ilu_Error errs;
    if (jjProtocolInfo) {
        cProtInfo = /*javaString.h*/ makeCString(jjProtocolInfo);
    } else {
        cProtInfo = ilu_DefaultProtocolInfo();
    }
    if (jjTransportInfo) {
        cTransInfo = Access(jjTransportInfo, yIluTransportInfo, ilu_TransportInfo);
    }
    if (cTransInfo == 0) {
        cTransInfo = ilu_DefaultTransportInfo();
    }
    cServer = Access(jjServer, yIluServer, ilu_Server);
    if (_ilujava_pctFlag>0) {
        ilu_DebugPrintf("$ IluPort_nativeInitPort jprt: %x jsrv: %x csrv %x\n", 
            (int) jjPort, (int) jjServer, (int) cServer);
    }
    if (jjPassport) {
        cPassport = Access(jjPassport, yIluPassport, ilu_Passport);
    }
    cPort = ilu_CreatePort(cServer, cProtInfo, cTransInfo, cPassport, &errs);
    if (ILU_ERRNOK(errs)) {
        BUGPRINT("$ nativeInitPort: Error creating port");
        _ilujava_IluErrorToException(&errs, "Error creating port");
        return;
    }
    if (cPort == 0) {
        BUGPRINT("$ nativeInitPort: null port");
        SignalError(EE(), "xerox/ilu/IluSomeSystemException", "null port");
        return;
    }
    Access(jjPort, yPort, ilu_Port) = cPort;
    if (_ilujava_pctFlag>0) {
        ilu_DebugPrintf("$ Port created %x \n", (int) cPort);
    }
} /* nativeInitPort */


EXPORTJAVA void 
xerox_ilu_IluPort_nativeClosePort(
		struct Hxerox_ilu_IluPort * jjPort
		)
{
    ilu_Port cPort = Access(jjPort, yPort, ilu_Port);
    if (cPort == 0) return;
    ilu_ClosePort(cPort);
 }  /* nativeClosePort */


EXPORTJAVA void 
xerox_ilu_IluPort_nativeDonePort(
		struct Hxerox_ilu_IluPort * jjPort
		)
{
    ILU_ERRS((bad_param, bad_locks, internal)) err;
    ilu_boolean dontCare;
    ilu_Port cPort = Access(jjPort, yPort, ilu_Port);
    ILU_CLER(err);
    if (cPort == 0) return;
    Access(jjPort, yPort, ilu_Port) = 0;
    if (unhand(jjPort)->pstate > 3) return;
    unhand(jjPort)->pstate = 99;
    dontCare = ilu_DoneWithPort(cPort, &err);
}  /* nativeDonePort */


EXPORTJAVA void 
xerox_ilu_IluTransportInfo_nativeInitTransportInfo(
		struct Hxerox_ilu_IluTransportInfo * jjTransInfo,
		HArrayOfString * jjInfo)
{
    char ** cObject = 0;
    if (jjInfo) {
        char * rover; /* the next un-assigned byte */
        struct Hjava_lang_String * jjString;
        int totalLen = 0;
        int thisLen = 0;
        int i;
        int cnt = obj_length(jjInfo);
        /* compute length */
        for (i = 0; i < cnt; i = i+1) {
            jjString = unhand(jjInfo)->body[i];
            thisLen = javaStringLength(jjString);/*javaString.h*/
            totalLen = totalLen+thisLen;
        }
        totalLen = totalLen + cnt; /* add padding per string */
        totalLen = totalLen + cnt; /* add extra padding per string; see below */
        totalLen = totalLen + (cnt+2)*sizeof(char *); /*pointer array, stopper and final padding*/
        /* allocate data */
        cObject = (char **) ilu_must_malloc(totalLen);
        /* copy data */
        cObject[cnt] = 0;
        rover = (char *) & (cObject[cnt+1]);
        for (i = 0; i < cnt; i = i+1) {
            cObject[i] = rover;
            jjString = unhand(jjInfo)->body[i];
            thisLen = javaStringLength(jjString);/*javaString.h*/
            /* 
             * ? Must add 1 otherwise it overwrites the last character.   
             * I do not understand this, but the java rt does the same thing.
             */
            javaString2CString(jjString, rover, thisLen+1); /*javaString.h*/
            rover = rover + thisLen+1;
            rover[0] = '\0';
            rover = rover + 1; 
        }
    }
    Access(jjTransInfo, yIluTransportInfo, char **) = cObject; 
} /* nativeInitTransportInfo */


EXPORTJAVA void 
xerox_ilu_IluTransportInfo_nativeFinalizeTransportInfo(
		struct Hxerox_ilu_IluTransportInfo * jjTransInfo)
{
    char * cMem = Access(jjTransInfo, yIluTransportInfo, char *);
    Access(jjTransInfo, yIluTransportInfo, char *) = 0;
    if (cMem) ilu_free(cMem);
} /* nativeFinalizeTransportInfo */


static ilu_Passport defaultPassport;


EXPORTLIMITED void
_ilujava_PassportInit()
{
    ilu_Error err = ILU_INIT_NO_ERR;
    defaultPassport = ilu_CreatePassport(0, &err);
}


EXPORTLIMITED ilu_Passport 
_ilujava_getIluPassport(
	ilu_refany jIluPassport /* struct Hxerox_ilu_IluPassport* */ 
	)
/* Converts java passport to ilu passport.
 * Converts null IluPassport into default passport. 
 */
 {
     ilu_Passport cpp = 0;
     struct Hxerox_ilu_IluPassport* jjpp 
     	= (struct Hxerox_ilu_IluPassport*) jIluPassport;
     if (jjpp==0) return defaultPassport;
     cpp = Access(jjpp, yIluPassport, ilu_Passport);
     if (cpp==0) return defaultPassport;
     return cpp;
 }


/* end */
