/* IluJava_Pickle.c */
/* Chris Jacobi, April 22, 1997 6:09 pm PDT */

/*
 * Copyright (c) 1996, 1997 Xerox Corporation.  All Rights Reserved.  
 * Unlimited use, reproduction, and distribution of this software is
 * permitted.  Any copy of this software must include both the above
 * copyright notice of Xerox Corporation and this paragraph.  Any
 * distribution of this software must comply with all applicable United
 * States export control laws.  This software is made available AS IS,
 * and XEROX CORPORATION DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
 * INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE, AND NOTWITHSTANDING ANY OTHER
 * PROVISION CONTAINED HEREIN, ANY LIABILITY FOR DAMAGES RESULTING FROM
 * THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 * CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 * XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
 
/* $Id: IluJava_Pickle.c,v 1.23 1997/04/23 06:27:51 jacobi Exp $ */

/*
 * C side for Pickle.java, TypeCode.java, and, IluTypeRep.java
 */
 
#include "IluJava_CallMacros.h"


#define JPICKLE  struct Hxerox_ilu_Pickle *
#define JTYPECODE  struct Hxerox_ilu_TypeCode *
#define JILUTYPEREP  struct Hxerox_ilu_IluTypeRep *

#define SystemException "xerox/ilu/IluSystemException"


static char*
J2Cstring8X(JString jjString)
/* restrict to 8 bits; allocated in ilu heap; return "" if jjString is null */
{
    char* cstring;
    int leng = 0;
    if (jjString) leng = /*javaString.h*/javaStringLength(jjString);
    cstring = ilu_must_malloc(leng+1);
    IluJava_JString_toC8(jjString, cstring, leng+1);
    return cstring;
}


static char*
J2Cstring8Y(JString jjString)
/* restrict to 8 bits; allocated in ilu heap; return NIL if jjString is null */
{
    char* cstring = 0;
    int leng = 0;
    if (jjString) {
        leng = /*javaString.h*/javaStringLength(jjString);
        cstring = ilu_must_malloc(leng+1);
        IluJava_JString_toC8(jjString, cstring, leng+1);
    }
    return cstring;
}

EXPORTJAVA void
xerox_ilu_Pickle_nativeOutPickle(JPICKLE jjPickle, JIluCall jjcall)
/* jjcall is null tested in java */
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    ilu_Pickle* cPicklePtr;
    PROLOG(CALL_OUT, ILUJAVA_VOID)
    cPicklePtr = Access(jjPickle, ypickleptr, ilu_Pickle*);
    if (cPicklePtr == 0) {
        SignalError(EE(), SystemException, "pickle not initialized");
        return;
    }
    ilu_OutputPickle(cCall, *cPicklePtr, &ioerrs);
    ERRCHECK(ILUJAVA_VOID)
#else
    BADCASE;
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}
 

EXPORTJAVA Jint
xerox_ilu_Pickle_nativeSzPickle(JPICKLE jjPickle, JIluCall jjcall)
/* jjcall is null tested in java */
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    ilu_Pickle* cPicklePtr;
    PROLOG_SZ()
    cPicklePtr = Access(jjPickle, ypickleptr, ilu_Pickle*);
    if (cPicklePtr == 0) {
        SignalError(EE(), SystemException, "pickle not initialized");
        return 0;
    }
    sz = ilu_SizeOfPickle(cCall, *cPicklePtr, &ioerrs);
    ERRCHECK(0)
    return (Jint) sz;
#else
    return 0;
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}


static ilu_Pickle* allocPickle(JPICKLE jjPickle)
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    ilu_Pickle* cPicklePtr = ilu_must_malloc(sizeof(ilu_Pickle));
    cPicklePtr->pi_len = 0;
    cPicklePtr->pi_bytes = 0;
    Access(jjPickle, ypickleptr, ilu_Pickle*) = cPicklePtr;
    return cPicklePtr;
#else
    return 0;
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_Pickle_nativeInPickle(JPICKLE jjPickle, JIluCall jjcall)
/* jjcall is null tested in java */
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    ilu_Pickle* cPicklePtr;
    PROLOG(CALL_IN, ILUJAVA_VOID)
    cPicklePtr = allocPickle(jjPickle);
    if (cPicklePtr==0) {
        SignalError(EE(), SystemException, "pickle not initialized");
        return;
    }
    ilu_InputPickle(cCall, cPicklePtr, &ioerrs);
    ERRCHECK(ILUJAVA_VOID)
#else
    BADCASE;
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_Pickle_nativeFinalizePickle(JPICKLE jjPickle, Jlong savedYpickleptr)
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    ilu_Error errs = ILU_INIT_NO_ERR;
    ilu_Pickle* cPicklePtr;
    Access(jjPickle, ypickleptr, ilu_Pickle*) = 0;
    cPicklePtr =  * (ilu_Pickle**) &savedYpickleptr;
    if (cPicklePtr) {
        ilu_FreePickle(cPicklePtr, &errs);
        ilu_free(cPicklePtr);
    }
    /* don't finalize the JIluCall here: it has its own finalization */
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_Pickle_nativeStartPickleInsert(JPICKLE jjPickle, 
    JIluCall jjcall)
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    ilu_Error errs = ILU_INIT_NO_ERR;
    ilu_boolean success;
    ilu_Call cCall;
    ilu_Pickle* cPicklePtr;
    cPicklePtr = Access(jjPickle, ypickleptr, ilu_Pickle*);
    if (cPicklePtr) {
        SignalError(EE(), SystemException, "Pickle already defined");
        return;
    }
    xerox_ilu_IluCall_initCallShell(jjcall);
    cCall = Access(jjcall, yCall, ilu_Call);
    success = ilu_StartPickle((ilu_Call_s *)cCall, &errs);
    if (ILU_ERRNOK(errs)) {
        _ilujava_IluErrorToException(&errs, "nativeStartPickle1"); 
        return;
    }
    allocPickle(jjPickle);
    unhand(jjcall)->jNeedsSizing = (Jboolean) ilu_CallNeedsSizing(cCall);
    TRANSITION(CALL_SZ)
    return;
#else
    BADCASE;
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_Pickle_nativeStartPickleExtract(JPICKLE jjPickle, 
    JIluCall jjcall, 
    JString jjuid_expected)
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    char* cuid_expected = 0;
    ilu_Error errs = ILU_INIT_NO_ERR;
    ilu_boolean success;
    ilu_Call cCall;
    ilu_Pickle* cPicklePtr;
    xerox_ilu_IluCall_initCallShell(jjcall);
    cCall = Access(jjcall, yCall, ilu_Call);
    success = ilu_StartPickle((ilu_Call_s *)cCall, &errs);
    if (ILU_ERRNOK(errs)) {
        _ilujava_IluErrorToException(&errs, "nativeStartPickle1"); 
        return;
    }
    cPicklePtr = Access(jjPickle, ypickleptr, ilu_Pickle*);
    if (cPicklePtr == 0) {
        SignalError(EE(), SystemException, "pickle finalized");
        return;
    }
    if (jjuid_expected) {
        ilu_string cuid_found;
        if (cPicklePtr == 0) {
            _ilujava_IluErrorToException(&errs, "Pickle not defined");
            return;
        }
        cuid_found = Access(jjPickle, yuuid, ilu_string);
        if (cuid_found == 0) {
            cuid_found = ilu_PickleType(*cPicklePtr, &errs);
            ILU_CLER(errs);
            if (ILU_ERRNOK(errs)) {
                _ilujava_IluErrorToException(&errs, "nativeStartPickle2");
                return;
            }
        }
        cuid_expected = J2Cstring8X(jjuid_expected);
        if (strcmp(cuid_expected, cuid_found))  {
            ilu_Class cClassFound;
            ilu_Class cClassExcpected;
            /* type uuid's are not equal */
            cClassExcpected = ilu_FindClassFromID(cuid_expected);
            if (cClassExcpected == 0) {
                /* CONFUSING BUT OK
                 * If the expected class came from the program
                 *    it is perfectly ok to raise an exception if its not known
                 * If this is simply from reading the pickle's uuid field
                 *    it would have matched the strcmp
                 */
                SignalError(EE(), SystemException, "expected class unknown");
                goto freeAndReturn;
            }
            cClassFound = ilu_FindClassFromID(cuid_found);
            if (cClassFound==0) {
                /* ?? UNFORTUNATE !!
                 * We would like to be able to accept some unknown subclass.
                 * Reading of objects of unknown classes is possible, but
                 * I don't know how to test 
                 *   a) whether it is an object type
                 *   b) whether the object type is a sub type
                 */
                SignalError(EE(), SystemException, "found class unknown");
                goto freeAndReturn;
            }
            if (!ilu_IsSubObjectType(cClassExcpected, cClassFound)) {
                SignalError(EE(), SystemException, "not a subclass");
                goto freeAndReturn;
            }            
        }
        ilu_free(cuid_expected);
    }
    ILU_CLER(errs);
    success = ilu_ReadPickle(cCall, *cPicklePtr, &errs); 
       /* passes pickle ownership */
    if (!success) {
       _ilujava_IluErrorToException(&errs, "failed");
       return;
    }
    TRANSITION(CALL_IN)
    return;
    
    /* error returns */
    freeAndReturn:
        if (cuid_expected) ilu_free(cuid_expected);
#else
    BADCASE;
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
} 


EXPORTJAVA void
xerox_ilu_Pickle_nativeEndPickleWork(JPICKLE jjPickle, 
    JIluCall jjcall)
    /* doesn't raise exceptions (if call!=null); important ! */
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    ilu_Error errs = ILU_INIT_NO_ERR;
    ilu_boolean success;
    ilu_Call cCall;
    ilu_Pickle* cPicklePtr;
    cPicklePtr = Access(jjPickle, ypickleptr, ilu_Pickle*);
    cCall = Access(jjcall, yCall, ilu_Call);
    if (cCall && cPicklePtr) {
       /* never free the memory here; we do that in the finalization */
       success = ilu_EndPickle(cCall, cPicklePtr, &errs);
    }
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_Pickle_nativeWritePickle(JPICKLE jjPickle, 
    JIluCall jjcall, 
    Jint sz, 
    JString jjuid)
/* no changes about memory ownership */
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    ilu_Error errs = ILU_INIT_NO_ERR;
    ilu_boolean success;
    ilu_Call cCall;
    ilu_string cuuid = J2Cstring8X(jjuid);
    cCall = Access(jjcall, yCall, ilu_Call);
    if (_ilujava_pickleFlag > 0) {
        ilu_DebugPrintf("$ nativeWritePickle %d<%s>\n", strlen(cuuid), cuuid);
    }
    success = ilu_WritePickle(cCall, (ilu_cardinal) sz, cuuid, &errs);
    ilu_free(cuuid);
    if (ILU_ERRNOK(errs)) {
        _ilujava_IluErrorToException(&errs, "nativeWritePickle"); 
        return;
    }
    TRANSITION(CALL_OUT)
#else
    BADCASE;
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}


EXPORTJAVA JString
xerox_ilu_Pickle_nativeUuidPickle(JPICKLE jjPickle)
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    JString jjString;
    ilu_Error errs = ILU_INIT_NO_ERR;
    ilu_string cuuid;
    ilu_Pickle* cPicklePtr;
    cPicklePtr = Access(jjPickle, ypickleptr, ilu_Pickle*);
    if (cPicklePtr == 0) {
        SignalError(EE(), SystemException, "pickle falsely finalized");
    }
    cuuid = ilu_PickleType(*cPicklePtr, &errs);
        /* if the memory would have been freed: no smash, only wrong result */
    if (ILU_ERRNOK(errs)) {
        _ilujava_IluErrorToException(&errs, "nativeUuidPickle"); 
        return 0;
    }
    Access(jjPickle, yuuid, ilu_string) = cuuid; 
        /* out parameter belongs to the pickle */
    if (_ilujava_pickleFlag > 0) {
        ilu_DebugPrintf("$ nativeUuidPickle %d<%s>\n", strlen(cuuid), cuuid);
    }
    jjString = /*javaString.h*/ makeJavaString((char*) cuuid, strlen(cuuid));
    return jjString;
#else
    BADCASE1;
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_TypeCode_nativeInitTypeCode(JTYPECODE jjTypeCode, JString jjuid)
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    ilu_string cuuid;
    cuuid = /*javaString.h*/ allocCString(jjuid);
    if (_ilujava_pickleFlag > 0) {
        ilu_DebugPrintf("$ nativeInitTypeCode %d<%s>\n", strlen(cuuid), cuuid);
    }
    Access(jjTypeCode, yuuid, ilu_string) = cuuid; 
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}


typedef enum {
    ctk_null, ctk_void,
    ctk_short, ctk_long, ctk_ushort, ctk_ulong,
    ctk_float, ctk_double, ctk_boolean, ctk_char,
    ctk_octet, ctk_any, ctk_TypeCode, ctk_Principal, ctk_objref,
    ctk_struct, ctk_union, ctk_enum, ctk_string,
    ctk_sequence, ctk_array, ctk_alias, ctk_except,
    ctk_longlong, ctk_ulonglong, ctk_longdouble,
    ctk_wchar, ctk_wstring, ctk_fixed
    } corba_TypeKind;
    
/* Not implemented in ILU for varying reasons:
 * null, void, TypeCode, Principal, except, wstring, fixed
 */
 
EXPORTJAVA void
xerox_ilu_TypeCode_nativeSpecialInitTypeCode(JTYPECODE jjTypeCode, 
    Jint corbaTypeKind)
{
#if (defined(ADD_VARIANT_SUPPORT) || defined(IIOP_PROTOCOL))
    ilu_string cuuid;
    JString jjuid;
    switch ((int) corbaTypeKind) {
        case (int) ctk_short:
            cuuid = (char*) ilu_TypeID_ilu_shortinteger;
            break;
        case (int) ctk_long:
            cuuid = (char*) ilu_TypeID_ilu_integer;
            break;
        case (int) ctk_ushort:
            cuuid = (char*) ilu_TypeID_ilu_shortcardinal;
            break;
        case (int) ctk_ulong:
            cuuid = (char*) ilu_TypeID_ilu_cardinal;
            break;
        case (int) ctk_string:
            cuuid = (char*) ilu_TypeID_ilu_CString;
            break;
        case (int) ctk_boolean:
            cuuid = (char*) ilu_TypeID_ilu_boolean;
            break;
        case (int) ctk_char:
            cuuid = (char*) ilu_TypeID_ilu_shortcharacter;
            break;
        case (int) ctk_wchar:
            cuuid = (char*) ilu_TypeID_ilu_character;
            break;
        case (int) ctk_float:
            cuuid = (char*) ilu_TypeID_ilu_shortreal;
            break;
        case (int) ctk_double:
            cuuid = (char*) ilu_TypeID_ilu_real;
            break;
        case (int) ctk_octet:
            cuuid = (char*) ilu_TypeID_ilu_byte;
            break;
        case (int) ctk_longlong:
            cuuid = (char*) ilu_TypeID_ilu_longinteger;
            break;
        case (int) ctk_ulonglong:
            cuuid = (char*) ilu_TypeID_ilu_longcardinal;
            break;
        case (int) ctk_objref:
            cuuid = (char*) ilu_TypeID_ilu_CORBA_Object;
            break;
        case (int) ctk_any:
            cuuid = (char*) ilu_TypeID_ilu_pickle;
            break;
    }
    jjuid = /*javaString.h*/ makeJavaString((char*) cuuid, strlen(cuuid));
    Access(jjTypeCode, yuuid, ilu_string) = cuuid; 
    unhand(jjTypeCode)->jjuid = jjuid; 
#endif /* ADD_VARIANT_SUPPORT || IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_IluTypeRep_nativeRegisterSome(
    JILUTYPEREP jjTypeRep, JString jjName, JString jjIslIfName,
    JString jjIslIfBrand, JString jjuid, Jint elCnt, 
    Jint ilutk, JString jjBasUID, JArrayOfInt jjdims)
{
#ifdef IIOP_PROTOCOL
    ilu_Error errs = ILU_INIT_NO_ERR;
    ilu_Error err0 = ILU_INIT_NO_ERR;
    ilu_boolean new = ilu_FALSE;
    ilu_boolean freemutex = ilu_FALSE;
    ilu_Type cIluType;
    ilu_string cName = (ilu_string) J2Cstring8Y(jjName);
    ilu_string cIslIfBrand = (ilu_string) J2Cstring8Y(jjIslIfBrand);
    ilu_string cIslIfName = (ilu_string) J2Cstring8Y(jjIslIfName);
    ilu_string cuid = (ilu_string) J2Cstring8Y(jjuid);
    ilu_string cBaseUID = 0;
    if (jjuid) cBaseUID = (ilu_string) J2Cstring8Y(jjBasUID);
    ilu_EnterMutex(ilu_otmu, &errs);
    if (ILU_ERRNOK(err0)) {
        _ilujava_IluErrorToException(&err0, completedNo "RegisterEnumeration");
        goto return_free;
    }
    if (ilutk == 15 /*ilu_optional_tk*/ ) {
        cIluType = ilu_RegisterOptionalType(
            cName,
            cIslIfName,
            cIslIfBrand,
            cuid,
            cBaseUID,
            &new, 
            &errs
            );
        freemutex = ilu_TRUE;
    } else if (ilutk == 16 /*ilu_alias_tk*/ ) {
        cIluType = ilu_RegisterAliasType(
            cName,
            cIslIfName,
            cIslIfBrand,
            cuid,
            cBaseUID,
            &new, 
            &errs
            );
        freemutex = ilu_TRUE;
    } else if (ilutk == 18 /*ilu_sequence_tk*/ ) {
        cIluType = ilu_RegisterSequenceType(
            cName,
            cIslIfName,
            cIslIfBrand,
            cuid,
            cBaseUID,
            elCnt, /*limit*/
            &new, 
            &errs
            );
        freemutex = ilu_TRUE;
    } else if (ilutk == 19 /*ilu_record_tk*/ ) {
        cIluType = ilu_RegisterRecordType(
            cName,
            cIslIfName,
            cIslIfBrand,
            cuid,
            elCnt,
            &new, 
            &errs
            );
        /* dont freemutex on success */
    } else if (ilutk == 20 /*ilu_array_tk*/ ) {
        int i;
        ilu_cardinal* x;
        freemutex = ilu_TRUE;
        if (elCnt==0) {
            SignalError(EE(), SystemException, "0 elements");
            goto return_free_mutex; 
        }
        x = ilu_malloc(elCnt*sizeof(ilu_cardinal));
        for (i = 0; i < elCnt; i = i+1) {
            x[i] = unhand(jjdims)->body[i];
        }
        cIluType = ilu_RegisterArrayType(
            cName,
            cIslIfName,
            cIslIfBrand,
            cuid,
            cBaseUID,
            elCnt, /*no of dimensions*/
            x,
            &new, 
            &errs
            );
        ilu_free(x);
    } else if (ilutk == 21 /*ilu_enumeration_tk*/ ) {
        cIluType = ilu_RegisterEnumerationType(
            cName,
            cIslIfName,
            cIslIfBrand,
            cuid,
            elCnt,
            &new, 
            &errs
            );
        /* dont freemutex on success */
    }
    if (new) {
        if (ILU_ERROK(errs)) {
            Access(jjTypeRep, yIluType, ilu_Type) = cIluType;
            /* in success case keep the mutex on return
             * for some types but not all !
             */
        } else {
            freemutex = ilu_TRUE;
        }
    } else {
        freemutex = ilu_TRUE;
    }
    return_free_mutex:
    if (freemutex) {
        ilu_ExitMutex(ilu_otmu, ilu_TRUE, &errs);
    }
    /*fall through*/
    return_free:
        if (cName) ilu_free(cName);
        if (cIslIfBrand) ilu_free(cIslIfBrand);
        if (cIslIfName) ilu_free(cIslIfName);
        if (cuid) ilu_free(cuid);
        if (cBaseUID) ilu_free(cBaseUID);
#endif /* def IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_IluTypeRep_nativeRegisterUnionBase(
    JILUTYPEREP jjTypeRep, JString jjName, JString jjIslIfName,
    JString jjIslIfBrand, JString jjuid, 
    JString jjDiscriminatUID, 
    Jint armCnt, Jint defaultArm, 
    Jint othersAllowed, Jint valueKind)
{
#ifdef IIOP_PROTOCOL
    ilu_cardinal funnyIluDefaultArm = 0;
    ilu_Error errs = ILU_INIT_NO_ERR;
    ilu_Error err0 = ILU_INIT_NO_ERR;
    ilu_boolean new = ilu_FALSE;
    ilu_boolean freemutex = ilu_FALSE;
    ilu_Type cIluType;
    ilu_string cName = (ilu_string) J2Cstring8Y(jjName);
    ilu_string cIslIfBrand = (ilu_string) J2Cstring8Y(jjIslIfBrand);
    ilu_string cIslIfName = (ilu_string) J2Cstring8Y(jjIslIfName);
    ilu_string cuid = (ilu_string) J2Cstring8Y(jjuid);
    ilu_string cDiscUID = (ilu_string) J2Cstring8Y(jjDiscriminatUID);
    ilu_EnterMutex(ilu_otmu, &errs);
    if (ILU_ERRNOK(err0)) {
        _ilujava_IluErrorToException(&err0, completedNo "RegisterEnumeration");
        goto return_free;
    }
    if (defaultArm >= 0 AND defaultArm < armCnt) {
        funnyIluDefaultArm = defaultArm+1;
    }
    cIluType = ilu_RegisterUnionType(
            cName,
            cIslIfName,
            cIslIfBrand,
            cuid,
            cDiscUID,
            armCnt,
            funnyIluDefaultArm, 
            (othersAllowed>0), 
            &new, 
            &errs
            );
    /* dont freemutex on success */
    if (new) {
        if (ILU_ERROK(errs)) {
            Access(jjTypeRep, yIluType, ilu_Type) = cIluType;
            /* in success case keep the mutex on return
             * for some types but not all !
             */
        } else {
            freemutex = ilu_TRUE;
        }
    } else {
        freemutex = ilu_TRUE;
    }
    if (freemutex) {
        ilu_ExitMutex(ilu_otmu, ilu_TRUE, &errs);
    }
    /*fall through*/
    return_free:
        if (cName) ilu_free(cName);
        if (cDiscUID) ilu_free(cDiscUID);
        if (cIslIfBrand) ilu_free(cIslIfBrand);
        if (cIslIfName) ilu_free(cIslIfName);
        if (cuid) ilu_free(cuid);
#endif /* def IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_IluTypeRep_nativeRegisterEnumerationElement(
    JILUTYPEREP jjTypeRep, Jint elNum, JString elIslName, Jint elVal)
{
#ifdef IIOP_PROTOCOL
    ilu_Error errs = ILU_INIT_NO_ERR;
    if (((int)elNum) < 0) {
        /*special locking case */
        ilu_ExitMutex(ilu_otmu, ilu_FALSE, &errs);
    } else {
        /* normal case */
        ilu_boolean success;
        ilu_string cName;
        ilu_Type cIluType = Access(jjTypeRep, yIluType, ilu_Type);
        if (! cIluType) return;  /*impossible*/
        cName = (ilu_string) J2Cstring8Y(elIslName);
        success = ilu_RegisterEnumerationElement(cIluType, 
            elNum, cName, elVal, &errs);
        ilu_free(cName);
    }
#endif /* def IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_IluTypeRep_nativeRegisterRecordField(
    JILUTYPEREP jjTypeRep, 
    Jint index, JString fldIslName, JString jjIslIfBrand)
{
#ifdef IIOP_PROTOCOL
    ilu_Error errs = ILU_INIT_NO_ERR;
    if (((int)index) < 0) {
        /*special locking case */
        ilu_ExitMutex(ilu_otmu, ilu_FALSE, &errs);
    } else {
        /* normal case */
        ilu_boolean success;
        ilu_string cName, cBrand;
        ilu_Type cIluType = Access(jjTypeRep, yIluType, ilu_Type);
        if (! cIluType) return;  /*impossible*/
        cName = (ilu_string) J2Cstring8Y(fldIslName);
        cBrand = (ilu_string) J2Cstring8Y(jjIslIfBrand);
        success = ilu_RegisterRecordField(cIluType, 
            index, cName, cBrand, &errs);
        ilu_free(cName);
        ilu_free(cBrand);
    }
#endif /* def IIOP_PROTOCOL */
}


EXPORTJAVA void
xerox_ilu_IluTypeRep_nativeRegisterUnionArm(
    JILUTYPEREP jjTypeRep, 
    Jint armNum, JString jjIslArmName, JString jjArmUID,
    Jint valnum, JArrayOfInt jjIvals, JArrayOfString jjSvals, 
    Jint valueKind)
{
#ifdef IIOP_PROTOCOL
    ilu_Error errs = ILU_INIT_NO_ERR;
    if (((int)valueKind) < 0) {
        /*special locking case */
        ilu_ExitMutex(ilu_otmu, ilu_FALSE, &errs);
    } else {
        /* normal case */
        ilu_ConstantValue_s cvMemory;
        ilu_ConstantValue cv;
        int i;
        int constval;
        ilu_string cstring = 0;
        ilu_string cArmName, cArmTypeUID;
        ilu_Type cIluType = Access(jjTypeRep, yIluType, ilu_Type);
        ilu_UnionArm arm;
        if (! cIluType) return;  /*impossible*/
        cArmName = (ilu_string) J2Cstring8Y(jjIslArmName);
        cArmTypeUID = (ilu_string) J2Cstring8Y(jjArmUID);
        arm = ilu_RegisterUnionArm(cIluType, 
            armNum, cArmName, cArmTypeUID, valnum, &errs);
        ilu_free(cArmName);
        ilu_free(cArmTypeUID);
        if (ILU_ERRNOK(errs)) {
            return;
        }
        cv = &cvMemory;
        cv->kind = (ilu_ConstantValueKind) valueKind;
        switch (cv->kind) {
            case ilu_byte_cvk: 
            case ilu_shortinteger_cvk: 
            case ilu_integer_cvk: 
            case ilu_shortcardinal_cvk: 
            case ilu_cardinal_cvk: 
            case ilu_boolean_cvk: 
                if (jjIvals == 0) goto error_return;
                if (valnum > (Jint)obj_length(jjIvals)) goto error_return;
                break;
            case ilu_enumeration_cvk: 
                if (jjSvals == 0) goto error_return;
                if (valnum > (Jint)obj_length(jjSvals)) goto error_return;
                break;
            default: /*error*/ 
                goto error_return;
        }
        for (i = 0; i < valnum; i = i+1) {
            ILU_CLER(errs);
            switch (cv->kind) {
                case ilu_byte_cvk: 
                    constval = unhand(jjIvals)->body[i];
                    cv->value.byte_val = constval; break;
                case ilu_shortinteger_cvk: 
                    constval = unhand(jjIvals)->body[i];
                    cv->value.shortinteger_val = constval; break;
                case ilu_integer_cvk: 
                    constval = unhand(jjIvals)->body[i];
                    cv->value.integer_val = constval; break;
                case ilu_shortcardinal_cvk: 
                    constval = unhand(jjIvals)->body[i];
                    cv->value.shortcardinal_val = constval; break;
                case ilu_cardinal_cvk: 
                    constval = unhand(jjIvals)->body[i];
                    cv->value.cardinal_val = constval; break;
                case ilu_boolean_cvk: 
                    constval = unhand(jjIvals)->body[i];
                    cv->value.boolean_val = (constval!=0); break;
                case ilu_enumeration_cvk: 
                    cstring = (ilu_string) J2Cstring8Y(unhand(jjSvals)->body[i]);
                    cv->value.enumeration_val = cstring; 
                    break;
                case ilu_shortreal_cvk: /*huh?*/
                case ilu_string_cvk: /*huh?*/
                case ilu_real_cvk: /*huh?*/
                default: /*error*/ goto error_return;
            }
            ilu_RegisterUnionArmValue(arm, i, cv, &errs);
            if (cstring) ilu_free(cstring);
        }
        /* fall through */
        error_return: return;
    }
    return;
#endif /* def IIOP_PROTOCOL */
}

/* end */
